// ADSampleDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ADSample.h"
#include "ADSampleDlg.h"
#include "aiousb.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// Task Globals

#pragma pack(push, 1)
	typedef struct
	{
		unsigned char
			ChannelRange[16],
			CalibMode,
			TrigMode,
			StartStopCH,
			Oversample,
			AIMUXChEx //AIMUX start and end address bit extension register.
		;
	} TUSBAI1664Config;
#pragma pack(pop)

unsigned long DeviceIndex = diOnly;
bool bCal = true;
TUSBAI1664Config Config;
double ChannelVolts[16];
int NumChannels, NextChannel;

#define AM_GOTDATA (WM_USER+0x99)
#define AM_ENDSTREAM (WM_USER+0x98)

void ADCallback(unsigned short *pBuf, long BufSize, long F, long Context); // prototype

/////////////////////////////////////////////////////////////////////////////
// CADSampleDlg dialog

CADSampleDlg::CADSampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CADSampleDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CADSampleDlg)
	m_calFName = _T("");
	m_bDifferential = FALSE;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CADSampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CADSampleDlg)
	DDX_Control(pDX, IDC_DIFFCHECK, m_DiffCheck);
	DDX_Control(pDX, IDC_STOP_BUTTON, m_StopButton);
	DDX_Control(pDX, IDC_VOLT_LIST, m_VoltList);
	DDX_Control(pDX, IDC_CAL_EDIT, m_CalEdit);
	DDX_Control(pDX, IDC_CAL_STATIC, m_CalStatic);
	DDX_Control(pDX, IDC_GO_BUTTON, m_goBTN);
	DDX_Control(pDX, IDC_RANGE_COMBO, m_rangeBox);
	DDX_Text(pDX, IDC_CAL_EDIT, m_calFName);
	DDX_Check(pDX, IDC_DIFFCHECK, m_bDifferential);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CADSampleDlg, CDialog)
	//{{AFX_MSG_MAP(CADSampleDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_CBN_SELCHANGE(IDC_RANGE_COMBO, OnSelchangeRangeCombo)
	ON_BN_CLICKED(IDC_GO_BUTTON, OnGoButton)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_DIFFCHECK, OnDiffCheckClick)
	ON_BN_CLICKED(IDC_STOP_BUTTON, OnStopButton)
	ON_MESSAGE(AM_GOTDATA, AM_GotData)
    ON_MESSAGE(AM_ENDSTREAM, AM_EndStream)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CADSampleDlg message handlers

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CADSampleDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CADSampleDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}



BOOL CADSampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	unsigned long ConfigBufSize;
	double Hz;

	if ( ADC_QueryCal(DeviceIndex) != ERROR_SUCCESS )
	{
		//this board doesn't have calibration
		bCal = false;
		m_CalStatic.ModifyStyle(WS_VISIBLE, 0, 0);
		m_CalEdit.ModifyStyle(WS_VISIBLE, 0, 0);
	}

	//Stop the counter, in case it was running.
	Hz = 0;
	CTR_StartOutputFreq(DeviceIndex, 0, &Hz);

	//Set the combo box to the first A/D range, then call OnSelchangeRangeCombo()
	//to set all the channels to that range.
	m_rangeBox.SetCurSel(0);
	OnSelchangeRangeCombo();

	ZeroMemory(&Config, sizeof(Config));
	Config.CalibMode = 0; // Take actual data, not internal calibration sources.
	Config.TrigMode = 0x05; // Scan selected channels each counter rising edge.
	Config.Oversample = 0; // No oversample.
	ConfigBufSize = sizeof(Config);
	ADC_SetConfig (DeviceIndex, &Config, &ConfigBufSize);

	m_calFName = ":AUTO:";

	UpdateData(false);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CADSampleDlg::OnSelchangeRangeCombo()
{
	int I;
	byte RangeCode;

	UpdateData(true);

	//The ranges in the combo box are listed in order, so that the index is equal
    //to the range code.
	RangeCode = m_rangeBox.GetCurSel();
	NumChannels = 16;
    if ( m_bDifferential )  { RangeCode |= 0x08;NumChannels = 8;}

	for (I = 0; I <= 0xF; I++)
		Config.ChannelRange[I] = RangeCode
	;

	m_VoltList.ResetContent();
}

void CADSampleDlg::OnDiffCheckClick() 
{
	OnSelchangeRangeCombo();
}

void CADSampleDlg::OnGoButton() 
{
	unsigned long Status, ConfigBufSize;
	double Hz;

	if ( m_bDifferential )
		Config.StartStopCH = 0x70; //Select all 8 differential channels, from 0 to 7.
	else
		Config.StartStopCH = 0xF0 //Select all 16 channels, from 0 to 15.
	;
	ConfigBufSize = sizeof(Config);
	ADC_SetConfig(DeviceIndex, &Config, &ConfigBufSize);

	if ( bCal )
	{
		Status = ADC_SetCal(DeviceIndex, m_calFName.GetBuffer(0));
		m_calFName.ReleaseBuffer();
	}

	Hz = 0.0;
    CTR_StartOutputFreq(DeviceIndex, 0, &Hz); // stop counters


	// -- Start ADC_continuous w/ callback
	Status = ADC_BulkContinuousCallbackStart(DeviceIndex, 1024, 64, (unsigned long)m_hWnd, ADCallback);
	if ( Status != ERROR_SUCCESS )
	{
	  MessageBox("error");// something bad happened
	}

	Hz = 1000.0;
	CTR_StartOutputFreq(DeviceIndex, 0, &Hz);


	m_CalEdit.EnableWindow(false);
	m_rangeBox.EnableWindow(false);
	m_goBTN.EnableWindow(false);
	m_StopButton.EnableWindow(true);
	m_DiffCheck.EnableWindow(false);
}

void CADSampleDlg::OnStopButton() 
{
	unsigned long EndStatus, IOStatus;
	// -- Stop ADC_continuous 
    EndStatus = ADC_BulkContinuousEnd(DeviceIndex, &IOStatus);


	m_CalEdit.EnableWindow(true);
	m_rangeBox.EnableWindow(true);
	m_goBTN.EnableWindow(true);
	m_StopButton.EnableWindow(false);
	m_DiffCheck.EnableWindow(true);
}

void CADSampleDlg::AM_GotData(LPVOID param) 
{
	int ChannelsUsed;

	CString ChannelData;

	if ( m_bDifferential )
		ChannelsUsed = 8;
	else
		ChannelsUsed = 16
	;

	m_VoltList.ResetContent();
	for ( int I = 0; I < ChannelsUsed; I++ )
	{
		ChannelData.Format("Ch %d: %.4f V", I, ChannelVolts[I]);
	    m_VoltList.AddString(ChannelData);
	}
}

void CADSampleDlg::AM_EndStream(LPVOID param)
{
}

void ADCallback(unsigned short *pBuf, long BufSize, long F, long Context)
{
	double Volts;
	int iSample;
	char Range;

  //Note: This is called from an alternate thread context.
  //(Specifically a worker thread spawned by the driver.)

  if (BufSize)
  {
    for (iSample=0; iSample < (BufSize / 2); iSample++) 
	{
      Volts = 1.0/65536.0 * *pBuf;
      pBuf++;

      Range = Config.ChannelRange[NextChannel];
      if ((Range & 1) != 0) Volts = Volts * 2 - 1;
      if ((Range & 2) == 0) Volts *= 2;
      if ((Range & 4) == 0) Volts *= 5;
      //Because the display can be slow, and this is called from a worker
      //thread, we just load each channel's last reading for display. Any
      //direct processing of the data, like a Fourier transform, would go
      //here instead.
      ChannelVolts[NextChannel] = Volts;

      NextChannel++;
      if (NextChannel >= NumChannels) NextChannel = 0;
    }

    //This tells the main thread new data is ready.
    PostMessage(HWND(Context), AM_GOTDATA, 0, 0);
  }
  
  //This tells the main thread we've got the last continuous callback packet.
  if (F & 2) PostMessage(HWND(Context), AM_ENDSTREAM, 0, 0);
}